"""gevent_openssl.SSL - gevent compatibility with OpenSSL.SSL (pyOpenSSL)
"""

import OpenSSL.SSL
from gevent.socket import wait_read, wait_write

_real_connection = OpenSSL.SSL.Connection


class Connection(object):
    """OpenSSL Connection wrapper
    """

    _reverse_mapping = _real_connection._reverse_mapping

    def __init__(self, context, sock):
        self._context = context
        self._sock = sock
        self._connection = _real_connection(context, sock)

    def __getattr__(self, attr):
        return getattr(self._connection, attr)

    def __iowait(self, io_func, *args, **kwargs):
        fd = self._sock.fileno()
        timeout = self._sock.gettimeout()
        while True:
            try:
                return io_func(*args, **kwargs)
            except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantX509LookupError):
                wait_read(fd, timeout=timeout)
            except OpenSSL.SSL.WantWriteError:
                wait_write(fd, timeout=timeout)

    def accept(self):
        sock, addr = self._sock.accept()
        return Connection(self._context, sock), addr

    def do_handshake(self):
        return self.__iowait(self._connection.do_handshake)

    def connect(self, *args, **kwargs):
        return self.__iowait(self._connection.connect, *args, **kwargs)

    def send(self, data, flags=0):
        return self.__send(self._connection.send, data, flags)

    def sendall(self, data, flags=0):
        # Note: all of the types supported by OpenSSL's Connection.sendall,
        # basestring, memoryview, and buffer, support len(...) and slicing,
        # so they are safe to use here.
        while len(data) > 0:
            res = self.send(data, flags)
            data = data[res:]

    def __send(self, send_method, data, flags=0):
        try:
            return self.__iowait(send_method, data, flags)
        except OpenSSL.SSL.SysCallError as e:
            if e[0] == -1 and not data:
                # errors when writing empty strings are expected and can be
                # ignored
                return 0
            raise

    def recv(self, bufsiz, flags=0):
        pending = self._connection.pending()
        if pending:
            return self._connection.recv(min(pending, bufsiz))
        try:
            return self.__iowait(self._connection.recv, bufsiz, flags)
        except OpenSSL.SSL.ZeroReturnError:
            return ''
        except OpenSSL.SSL.SysCallError as e:
            if e[0] == -1 and 'Unexpected EOF' in e[1]:
                # errors when reading empty strings are expected and can be
                # ignored
                return ''
            raise

    def shutdown(self):
        try:
            return self.__iowait(self._connection.shutdown)
        except OpenSSL.SSL.SysCallError as e:
            # PyOpenSSL will raise an EPIPE if the connection was already
            # closed, and that's safe to ignore here.
            if e[1] == 'EPIPE':
                return True
            raise



from __future__ import print_function

from gevent import monkey
monkey.patch_all(thread=False)
import gevent
import gevent.socket

import gevent_openssl
gevent_openssl.monkey_patch()

import os
import socket
import time
from OpenSSL import SSL


def lame_connection_test():
    context = SSL.Context(SSL.TLSv1_METHOD)
    sock = socket.create_connection(('www.google.com', 443), timeout=2)
    connection = SSL.Connection(context, sock)
    connection.set_connect_state()
    connection.do_handshake() # XXX not needed?
    connection.sendall('GET / HTTP/1.1\r\n\r\n')
    resp = connection.recv(1024)
    if "The document has moved" in resp:
        print("Connection test OK")
    else:
        print("Connection test FAIL. Got: %r" % resp)


def lame_timeout_test():
    context = SSL.Context(SSL.TLSv1_METHOD)
    sock = socket.create_connection(('slashdot.org', 80), timeout=1)
    connection = SSL.Connection(context, sock)
    connection.set_connect_state() # XXX not needed?
    t0 = time.time()
    try:
        connection.do_handshake()
    except socket.timeout:
        print("Timeout test OK (%s)" % (time.time() - t0))
    else:
        print("Timeout test FAIL")

def here(p):
    return os.path.join(os.path.dirname(__file__), p)

def lame_server_test():
    context = SSL.Context(SSL.TLSv1_METHOD)
    context.use_privatekey_file(here('server.key'))
    context.use_certificate_file(here('server.cer'))
    context.load_verify_locations(here('root.cer'))
    server = SSL.Connection(context, socket.socket(socket.AF_INET, socket.SOCK_STREAM))
    server.bind(('localhost', 0))
    port = server.getsockname()[1]

    def run_server():
        server.listen(3)
        conn, _ = server.accept()
        conn.set_accept_state()
        conn.send("foo")
        conn.shutdown()
        conn.close()
        server.close()

    def run_client():
        context = SSL.Context(SSL.TLSv1_METHOD)
        sock = socket.create_connection(('localhost', port), timeout=2)
        conn = SSL.Connection(context, sock)
        conn.set_connect_state()
        resp = conn.recv(32)
        conn.shutdown()
        conn.close()
        print("Server test " + "OK" if resp == "foo" else "FAIL")

    gserver = gevent.spawn(run_server)
    gclient = gevent.spawn(run_client)
    gevent.joinall([gserver, gclient])

def test():
    tests = [
        lame_connection_test,
        lame_timeout_test,
        lame_server_test,
    ]
    gevent.joinall([gevent.spawn(t) for t in tests])

if __name__ == '__main__':
    test()



"""gevent_openssl - gevent compatibility with pyOpenSSL.

Usage
-----

Instead of importing OpenSSL directly, do so in the following manner:

..

    import gevent_openssl as OpenSSL

or

..

    import gevent_openssl; gevent_openssl.monkey_patch()

Any calls that would have blocked the current thread will now only block the
current green thread.

This compatibility is accomplished by ensuring the nonblocking flag is
set before any blocking operation and the OpenSSL file descriptor is
polled internally to trigger needed events.
"""

from OpenSSL import *
import gevent_openssl.SSL as SSL


def monkey_patch():
    """
    Monkey patches `OpenSSL.SSL.Connection`
    """
    mod = __import__('OpenSSL').SSL
    mod.Connection = SSL.Connection



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='gevent_openssl',
    version='1.2-custom1',
    description='A gevent wrapper for pyOpenSSL',
    author='Menno Finlay-Smits',
    author_email='menno@freshfoo.com',
    url='https://github.com/mjs/gevent_openssl',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    license='New BSD',
    platforms='any',
    install_requires=['pyOpenSSL>=0.11', 'gevent'],
)
